@@ -32,6 +32,7 @@ import numpy as np
 
 import matplotlib.pyplot as plt
 import matplotlib.dates as mdates
+from matplotlib.lines import Line2D
 import oclock
 
 from .general import NumericalGraphBase, UpdateGraphBase
@@ -76,6 +77,8 @@ class NumericalGraph(NumericalGraphBase):
                  names,
                  data_types,
                  colors=None,
+                 legends=None,
+                 linestyles=None,
                  linestyle='.',
                  data_as_array=False,
                  time_conversion='numpy'):
@@ -89,6 +92,16 @@ class NumericalGraph(NumericalGraphBase):
                       (dict can have more keys than those in 'names')
         - colors: optional dict of colors with keys 'fig', 'ax', and the
                   names of the recordings.
+        - legends: optional dict of legend names (iterable) corresponding to
+                   all channels of each sensor, with the names of the
+                   recordings as keys.
+        - linestyles: optional dict of linestyles (iterable) to distinguish
+                      channels and sensors, with the names of the recordings
+                      as keys. If not specified (None), all lines have the
+                      linestyle defined by the `linestyle=` parameter (see
+                      below). If only some recordings are specified, the other
+                      recordings have the default linestyle or the linestyle
+                      defined by the `linestyle=` parameter.
         - linestyle: Matplotlib linestyle (e.g. '.', '-', '.-' etc.)
         - data_as_array: if sensors return arrays of values for different times
                          instead of values for a single time, put this
@@ -101,6 +114,8 @@ class NumericalGraph(NumericalGraphBase):
         super().__init__(names=names,
                          data_types=data_types,
                          colors=colors,
+                         legends=legends,
+                         linestyles=linestyles,
                          linestyle=linestyle,
                          data_as_array=data_as_array)
 
